@@ -18,15 +18,11 @@
 
 #include "mdbtools.h"
 
-#define _XOPEN_SOURCE
 #include <time.h>
 
 #define OFFSET_MASK 0x1fff
 #define OLE_BUFFER_SIZE (MDB_BIND_SIZE*64)
 
-char *mdb_money_to_string(MdbHandle *mdb, int start);
-char *mdb_numeric_to_string(MdbHandle *mdb, int start, int prec, int scale);
-
 static int _mdb_attempt_bind(MdbHandle *mdb, 
 	MdbColumn *col, unsigned char isnull, int offset, int len);
 static char *mdb_date_to_string(MdbHandle *mdb, const char *fmt, void *buf, int start);
@@ -34,6 +30,17 @@ static char *mdb_date_to_string(MdbHandle *mdb, const char *fmt, void *buf, int
 static size_t mdb_copy_ole(MdbHandle *mdb, void *dest, int start, int size);
 #endif
 
+#ifndef HAVE_REALLOCF
+static void *reallocf(void *ptr, size_t len) {
+	void *ptr2 = realloc(ptr, len);
+	if (!ptr2) {
+		free(ptr);
+		return NULL;
+	}
+	return ptr2;
+}
+#endif
+
 static const int noleap_cal[] = {0,31,59,90,120,151,181,212,243,273,304,334,365};
 static const int leap_cal[]   = {0,31,60,91,121,152,182,213,244,274,305,335,366};
 
@@ -77,6 +84,8 @@ int mdb_bind_column(MdbTableDef *table, int col_num, void *bind_ptr, int *len_pt
 {
 	MdbColumn *col = NULL;
 
+	if (!table->columns)
+		return -1;
 	/* 
 	** the column arrary is 0 based, so decrement to get 1 based parameter 
 	*/
@@ -104,6 +113,9 @@ mdb_bind_column_by_name(MdbTableDef *table, gchar *col_name, void *bind_ptr, int
 	unsigned int i;
 	int col_num = -1;
 	MdbColumn *col;
+
+	if (!table->columns)
+		return -1;
 	
 	for (i=0;i<table->num_cols;i++) {
 		col=g_ptr_array_index(table->columns,i);
@@ -304,7 +316,7 @@ int mdb_read_row(MdbTableDef *table, unsigned int row)
 	MdbField *fields;
 	int num_fields;
 
-	if (table->num_cols == 0)
+	if (table->num_cols == 0 || !table->columns)
 		return 0;
 
 	if (mdb_find_row(mdb, row, &row_start, &row_size) == -1 || row_size == 0) {
@@ -395,6 +407,8 @@ int mdb_read_next_dpg(MdbTableDef *table)
 			break; /* unknow map type: goto fallback */
 		if (!next_pg)
 			return 0;
+		if ((guint32)next_pg == table->cur_phys_pg)
+			return 0; /* Infinite loop */
 
 		if (!mdb_read_pg(mdb, next_pg)) {
 			fprintf(stderr, "error: reading page %d failed.\n", next_pg);
@@ -449,6 +463,8 @@ mdb_fetch_row(MdbTableDef *table)
 	do {
 		if (table->is_temp_table) {
 			GPtrArray *pages = table->temp_table_pages;
+			if (pages->len == 0)
+				return 0;
 			rows = mdb_get_int16(
 				g_ptr_array_index(pages, table->cur_pg_num-1),
 				fmt->row_count_offset);
@@ -660,7 +676,10 @@ mdb_ole_read_full(MdbHandle *mdb, MdbColumn *col, size_t *size)
 	while ((len = mdb_ole_read_next(mdb, col, ole_ptr))) {
 		if (pos+len >= result_buffer_size) {
 			result_buffer_size += OLE_BUFFER_SIZE;
-			result = realloc(result, result_buffer_size);
+			if ((result = reallocf(result, result_buffer_size)) == NULL) {
+				fprintf(stderr, "Out of memory while reading OLE object\n");
+				return NULL;
+			}
 		}
 		memcpy(result + pos, col->bind_ptr, len);
 		pos += len;
@@ -979,12 +998,8 @@ char *mdb_col_to_string(MdbHandle *mdb, void *buf, int start, int datatype, int
 	double td;
 
 	switch (datatype) {
-		case MDB_BOOL:
-			/* shouldn't happen.  bools are handled specially
-			** by mdb_xfer_bound_bool() */
-		break;
 		case MDB_BYTE:
-			text = g_strdup_printf("%d", mdb_get_byte(buf, start));
+			text = g_strdup_printf("%hhd", mdb_get_byte(buf, start));
 		break;
 		case MDB_INT:
 			text = g_strdup_printf("%hd",
@@ -992,8 +1007,8 @@ char *mdb_col_to_string(MdbHandle *mdb, void *buf, int start, int datatype, int
 		break;
 		case MDB_LONGINT:
 		case MDB_COMPLEX:
-			text = g_strdup_printf("%ld",
-				mdb_get_int32(buf, start));
+			text = g_strdup_printf("%d",
+				(int)mdb_get_int32(buf, start));
 		break;
 		case MDB_FLOAT:
 			tf = mdb_get_single(buf, start);
@@ -1001,7 +1016,7 @@ char *mdb_col_to_string(MdbHandle *mdb, void *buf, int start, int datatype, int
 		break;
 		case MDB_DOUBLE:
 			td = mdb_get_double(buf, start);
-			text = g_strdup_printf("%.16g", td);
+			text = g_strdup_printf("%.16lg", td);
 		break;
 		case MDB_BINARY:
 			if (size<0) {
@@ -1029,12 +1044,14 @@ char *mdb_col_to_string(MdbHandle *mdb, void *buf, int start, int datatype, int
 		break;
 		case MDB_MONEY:
 			text = mdb_money_to_string(mdb, start);
-		case MDB_NUMERIC:
 		break;
 		case MDB_REPID:
-		  text = mdb_uuid_to_string(mdb->pg_buf, start);
+			text = mdb_uuid_to_string(buf, start);
 		break;
 		default:
+			/* shouldn't happen.  bools are handled specially
+			** by mdb_xfer_bound_bool() */
+			fprintf(stderr, "Warning: mdb_col_to_string called on unsupported data type %d.\n", datatype);
 			text = g_strdup("");
 		break;
 	}
