@@ -17,6 +17,7 @@
  */
 
 #include "mdbtools.h"
+#include "mdbprivate.h"
 #ifdef HAVE_LIBMSWSTR
 #include <mswstr/mswstr.h>
 #endif
@@ -238,7 +239,11 @@ mdb_read_indices(MdbTableDef *table)
 	table->num_real_idxs = 0;
 	tmpbuf = g_malloc(idx2_sz);
 	for (i=0;i<table->num_idxs;i++) {
-		read_pg_if_n(mdb, tmpbuf, &cur_pos, idx2_sz);
+		if (!read_pg_if_n(mdb, tmpbuf, &cur_pos, idx2_sz)) {
+			g_free(tmpbuf);
+			mdb_free_indices(table->indices);
+			return table->indices = NULL;
+		}
                 //fprintf(stderr, "Index defn: ");
                 //hexdump((unsigned char *)tmpbuf, idx2_sz);
 		pidx = g_malloc0(sizeof(MdbIndex));
@@ -273,8 +278,8 @@ mdb_read_indices(MdbTableDef *table)
 			name_sz=read_pg_if_16(mdb, &cur_pos);
 		}
 		tmpbuf = g_malloc(name_sz);
-		read_pg_if_n(mdb, tmpbuf, &cur_pos, name_sz);
-		mdb_unicode2ascii(mdb, tmpbuf, name_sz, pidx->name, sizeof(pidx->name));
+		if (read_pg_if_n(mdb, tmpbuf, &cur_pos, name_sz))
+			mdb_unicode2ascii(mdb, tmpbuf, name_sz, pidx->name, sizeof(pidx->name));
 		g_free(tmpbuf);
 		//fprintf(stderr, "index %d type %d name %s\n", pidx->index_num, pidx->index_type, pidx->name);
 	}
@@ -520,7 +525,7 @@ mdb_index_test_sargs(MdbHandle *mdb, MdbIndex *idx, char *buf, int len)
 			col->idx_sarg_cache = g_ptr_array_new();
 			for (j=0;j<col->num_sargs;j++) {
 				sarg = g_ptr_array_index (col->sargs, j);
-				idx_sarg = g_memdup(sarg,sizeof(MdbSarg));
+				idx_sarg = g_memdup2(sarg,sizeof(MdbSarg));
 				//printf("calling mdb_index_cache_sarg\n");
 				mdb_index_cache_sarg(col, sarg, idx_sarg);
 				g_ptr_array_add(col->idx_sarg_cache, idx_sarg);
